@@ -1,8 +1,10 @@
 import hashlib
 import random
 
+from django import forms
 from django.core.mail import send_mail
 from django.template.loader import render_to_string
+from django.utils.translation import gettext_lazy as _
 
 from account.conf import settings
 
@@ -39,7 +41,10 @@ class AccountDefaultHookSet(object):
         return hash_func("".join(bits).encode("utf-8")).hexdigest()
 
     def generate_signup_code_token(self, email=None):
-        return self.generate_random_token([email])
+        extra = []
+        if email:
+            extra.append(email)
+        return self.generate_random_token(extra)
 
     def generate_email_confirmation_token(self, email):
         return self.generate_random_token([email])
@@ -50,6 +55,18 @@ class AccountDefaultHookSet(object):
             "password": form.cleaned_data["password"],
         }
 
+    def clean_password(self, password_new, password_new_confirm):
+        if password_new != password_new_confirm:
+            raise forms.ValidationError(_("You must type the same password each time."))
+        return password_new
+
+    def account_delete_mark(self, deletion):
+        deletion.user.is_active = False
+        deletion.user.save()
+
+    def account_delete_expunge(self, deletion):
+        deletion.user.delete()
+
 
 class HookProxy(object):
 
