@@ -1,6 +1,3 @@
-from __future__ import print_function
-from __future__ import unicode_literals
-
 from django.core.management.base import BaseCommand
 
 from account.models import AccountDeletion
