@@ -1,55 +1,48 @@
-from __future__ import unicode_literals
-
 import datetime
+import functools
 import operator
+from urllib.parse import urlencode
 
-try:
-    from urllib.parse import urlencode
-except ImportError:  # python 2
-    from urllib import urlencode
-
-from django.core.urlresolvers import reverse
+from django import forms
+from django.contrib.auth.models import AnonymousUser
+from django.contrib.sites.models import Site
 from django.db import models, transaction
 from django.db.models import Q
 from django.db.models.signals import post_save
-from django.utils import timezone, translation, six
-from django.utils.translation import ugettext_lazy as _
-
-from django.contrib.auth.models import AnonymousUser
-from django.contrib.sites.models import Site
+from django.dispatch import receiver
+from django.urls import reverse
+from django.utils import timezone, translation
+from django.utils.translation import gettext_lazy as _
 
 import pytz
-
 from account import signals
-from account.compat import AUTH_USER_MODEL, receiver
 from account.conf import settings
 from account.fields import TimeZoneField
 from account.hooks import hookset
+from account.languages import DEFAULT_LANGUAGE
 from account.managers import EmailAddressManager, EmailConfirmationManager
 from account.signals import signup_code_sent, signup_code_used
 
 
 class Account(models.Model):
 
-    user = models.OneToOneField(AUTH_USER_MODEL, related_name="account", verbose_name=_("user"))
+    user = models.OneToOneField(settings.AUTH_USER_MODEL, related_name="account", verbose_name=_("user"), on_delete=models.CASCADE)
     timezone = TimeZoneField(_("timezone"))
     language = models.CharField(
         _("language"),
         max_length=10,
         choices=settings.ACCOUNT_LANGUAGES,
-        default=settings.LANGUAGE_CODE
+        default=DEFAULT_LANGUAGE,
     )
 
     @classmethod
     def for_request(cls, request):
-        if request.user.is_authenticated():
-            try:
-                account = Account._default_manager.get(user=request.user)
-            except Account.DoesNotExist:
-                account = AnonymousAccount(request)
-        else:
-            account = AnonymousAccount(request)
-        return account
+        user = getattr(request, "user", None)
+        if user and user.is_authenticated:
+            account = user.account
+            if account:
+                return account
+        return AnonymousAccount(request)
 
     @classmethod
     def create(cls, request=None, **kwargs):
@@ -58,7 +51,7 @@ class Account(models.Model):
         account = cls(**kwargs)
         if "language" not in kwargs:
             if request is None:
-                account.language = settings.LANGUAGE_CODE
+                account.language = DEFAULT_LANGUAGE
             else:
                 account.language = translation.get_language_from_request(request, check_path=True)
         account.save()
@@ -77,21 +70,21 @@ class Account(models.Model):
         Returns a timezone aware datetime localized to the account's timezone.
         """
         now = datetime.datetime.utcnow().replace(tzinfo=pytz.timezone("UTC"))
-        timezone = settings.TIME_ZONE if not self.timezone else self.timezone
-        return now.astimezone(pytz.timezone(timezone))
+        tz = settings.TIME_ZONE if not self.timezone else self.timezone
+        return now.astimezone(pytz.timezone(tz))
 
     def localtime(self, value):
         """
         Given a datetime object as value convert it to the timezone of
         the account.
         """
-        timezone = settings.TIME_ZONE if not self.timezone else self.timezone
+        tz = settings.TIME_ZONE if not self.timezone else self.timezone
         if value.tzinfo is None:
             value = pytz.timezone(settings.TIME_ZONE).localize(value)
-        return value.astimezone(pytz.timezone(timezone))
+        return value.astimezone(pytz.timezone(tz))
 
 
-@receiver(post_save, sender=AUTH_USER_MODEL)
+@receiver(post_save, sender=settings.AUTH_USER_MODEL)
 def user_post_save(sender, **kwargs):
     """
     After User.save is called we check to see if it was a created user. If so,
@@ -101,23 +94,28 @@ def user_post_save(sender, **kwargs):
     We only run on user creation to avoid having to check for existence on
     each call to User.save.
     """
+
+    # Disable post_save during manage.py loaddata
+    if kwargs.get("raw", False):
+        return False
+
     user, created = kwargs["instance"], kwargs["created"]
     disabled = getattr(user, "_disable_account_creation", not settings.ACCOUNT_CREATE_ON_SAVE)
     if created and not disabled:
         Account.create(user=user)
 
 
-class AnonymousAccount(object):
+class AnonymousAccount:
 
     def __init__(self, request=None):
         self.user = AnonymousUser()
         self.timezone = settings.TIME_ZONE
         if request is None:
-            self.language = settings.LANGUAGE_CODE
+            self.language = DEFAULT_LANGUAGE
         else:
             self.language = translation.get_language_from_request(request, check_path=True)
 
-    def __unicode__(self):
+    def __str__(self):
         return "AnonymousAccount"
 
 
@@ -132,8 +130,8 @@ class SignupCode(models.Model):
     code = models.CharField(_("code"), max_length=64, unique=True)
     max_uses = models.PositiveIntegerField(_("max uses"), default=0)
     expiry = models.DateTimeField(_("expiry"), null=True, blank=True)
-    inviter = models.ForeignKey(AUTH_USER_MODEL, null=True, blank=True)
-    email = models.EmailField(blank=True)
+    inviter = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, on_delete=models.CASCADE)
+    email = models.EmailField(max_length=254, blank=True)
     notes = models.TextField(_("notes"), blank=True)
     sent = models.DateTimeField(_("sent"), null=True, blank=True)
     created = models.DateTimeField(_("created"), default=timezone.now, editable=False)
@@ -143,7 +141,7 @@ class SignupCode(models.Model):
         verbose_name = _("signup code")
         verbose_name_plural = _("signup codes")
 
-    def __unicode__(self):
+    def __str__(self):
         if self.email:
             return "{0} [{1}]".format(self.email, self.code)
         else:
@@ -155,8 +153,10 @@ class SignupCode(models.Model):
         if code:
             checks.append(Q(code=code))
         if email:
-            checks.append(Q(email=code))
-        return cls._default_manager.filter(six.moves.reduce(operator.or_, checks)).exists()
+            checks.append(Q(email=email))
+        if not checks:
+            return False
+        return cls._default_manager.filter(functools.reduce(operator.or_, checks)).exists()
 
     @classmethod
     def create(cls, **kwargs):
@@ -207,7 +207,7 @@ class SignupCode(models.Model):
         signup_code_used.send(sender=result.__class__, signup_code_result=result)
 
     def send(self, **kwargs):
-        protocol = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http")
+        protocol = settings.ACCOUNT_DEFAULT_HTTP_PROTOCOL
         current_site = kwargs["site"] if "site" in kwargs else Site.objects.get_current()
         if "signup_url" not in kwargs:
             signup_url = "{0}://{1}{2}?{3}".format(
@@ -232,8 +232,8 @@ class SignupCode(models.Model):
 
 class SignupCodeResult(models.Model):
 
-    signup_code = models.ForeignKey(SignupCode)
-    user = models.ForeignKey(AUTH_USER_MODEL)
+    signup_code = models.ForeignKey(SignupCode, on_delete=models.CASCADE)
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
     timestamp = models.DateTimeField(default=timezone.now)
 
     def save(self, **kwargs):
@@ -243,8 +243,8 @@ class SignupCodeResult(models.Model):
 
 class EmailAddress(models.Model):
 
-    user = models.ForeignKey(AUTH_USER_MODEL)
-    email = models.EmailField(unique=settings.ACCOUNT_EMAIL_UNIQUE)
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
+    email = models.EmailField(max_length=254, unique=settings.ACCOUNT_EMAIL_UNIQUE)
     verified = models.BooleanField(_("verified"), default=False)
     primary = models.BooleanField(_("primary"), default=False)
 
@@ -256,7 +256,7 @@ class EmailAddress(models.Model):
         if not settings.ACCOUNT_EMAIL_UNIQUE:
             unique_together = [("user", "email")]
 
-    def __unicode__(self):
+    def __str__(self):
         return "{0} ({1})".format(self.email, self.user)
 
     def set_as_primary(self, conditional=False):
@@ -281,7 +281,7 @@ class EmailAddress(models.Model):
         """
         Given a new email address, change self and re-confirm.
         """
-        with transaction.commit_on_success():
+        with transaction.atomic():
             self.user.email = new_email
             self.user.save()
             self.email = new_email
@@ -290,11 +290,21 @@ class EmailAddress(models.Model):
             if confirm:
                 self.send_confirmation()
 
+    def validate_unique(self, exclude=None):
+        super(EmailAddress, self).validate_unique(exclude=exclude)
+
+        qs = EmailAddress.objects.filter(email__iexact=self.email)
+
+        if qs.exists() and settings.ACCOUNT_EMAIL_UNIQUE:
+            raise forms.ValidationError({
+                "email": _("A user is registered with this email address."),
+            })
+
 
 class EmailConfirmation(models.Model):
 
-    email_address = models.ForeignKey(EmailAddress)
-    created = models.DateTimeField(default=timezone.now())
+    email_address = models.ForeignKey(EmailAddress, on_delete=models.CASCADE)
+    created = models.DateTimeField(default=timezone.now)
     sent = models.DateTimeField(null=True)
     key = models.CharField(max_length=64, unique=True)
 
@@ -304,7 +314,7 @@ class EmailConfirmation(models.Model):
         verbose_name = _("email confirmation")
         verbose_name_plural = _("email confirmations")
 
-    def __unicode__(self):
+    def __str__(self):
         return "confirmation for {0}".format(self.email_address)
 
     @classmethod
@@ -328,7 +338,7 @@ class EmailConfirmation(models.Model):
 
     def send(self, **kwargs):
         current_site = kwargs["site"] if "site" in kwargs else Site.objects.get_current()
-        protocol = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http")
+        protocol = settings.ACCOUNT_DEFAULT_HTTP_PROTOCOL
         activate_url = "{0}://{1}{2}".format(
             protocol,
             current_site.domain,
@@ -349,8 +359,8 @@ class EmailConfirmation(models.Model):
 
 class AccountDeletion(models.Model):
 
-    user = models.ForeignKey(AUTH_USER_MODEL, null=True, blank=True, on_delete=models.SET_NULL)
-    email = models.EmailField()
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, on_delete=models.SET_NULL)
+    email = models.EmailField(max_length=254)
     date_requested = models.DateTimeField(_("date requested"), default=timezone.now)
     date_expunged = models.DateTimeField(_("date expunged"), null=True, blank=True)
 
@@ -365,7 +375,7 @@ class AccountDeletion(models.Model):
         before = timezone.now() - datetime.timedelta(hours=hours_ago)
         count = 0
         for account_deletion in cls.objects.filter(date_requested__lt=before, user__isnull=False):
-            settings.ACCOUNT_DELETION_EXPUNGE_CALLBACK(account_deletion)
+            hookset.account_delete_expunge(account_deletion)
             account_deletion.date_expunged = timezone.now()
             account_deletion.save()
             count += 1
@@ -376,5 +386,26 @@ class AccountDeletion(models.Model):
         account_deletion, created = cls.objects.get_or_create(user=user)
         account_deletion.email = user.email
         account_deletion.save()
-        settings.ACCOUNT_DELETION_MARK_CALLBACK(account_deletion)
+        hookset.account_delete_mark(account_deletion)
         return account_deletion
+
+
+class PasswordHistory(models.Model):
+    """
+    Contains single password history for user.
+    """
+    class Meta:
+        verbose_name = _("password history")
+        verbose_name_plural = _("password histories")
+
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="password_history", on_delete=models.CASCADE)
+    password = models.CharField(max_length=255)  # encrypted password
+    timestamp = models.DateTimeField(default=timezone.now)  # password creation time
+
+
+class PasswordExpiry(models.Model):
+    """
+    Holds the password expiration period for a single user.
+    """
+    user = models.OneToOneField(settings.AUTH_USER_MODEL, related_name="password_expiry", verbose_name=_("user"), on_delete=models.CASCADE)
+    expiry = models.PositiveIntegerField(default=0)
