@@ -1,30 +1,126 @@
-from __future__ import unicode_literals
-
+from django.contrib import auth, messages
+from django.contrib.auth import get_user_model
+from django.contrib.auth.hashers import make_password
+from django.contrib.auth.tokens import default_token_generator
+from django.contrib.sites.shortcuts import get_current_site
+from django.core.exceptions import ImproperlyConfigured
 from django.http import Http404, HttpResponseForbidden
-from django.shortcuts import redirect, get_object_or_404
+from django.shortcuts import get_object_or_404, redirect
+from django.urls import reverse
+from django.utils.decorators import method_decorator
 from django.utils.http import base36_to_int, int_to_base36
-from django.core.urlresolvers import reverse
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
+from django.views.decorators.cache import never_cache
+from django.views.decorators.csrf import csrf_protect
+from django.views.decorators.debug import sensitive_post_parameters
 from django.views.generic.base import TemplateResponseMixin, View
 from django.views.generic.edit import FormView
 
-from django.contrib import auth, messages
-from django.contrib.sites.models import get_current_site
-from django.contrib.auth.tokens import default_token_generator
-
 from account import signals
-from account.compat import get_user_model
 from account.conf import settings
-from account.forms import SignupForm, LoginUsernameForm
-from account.forms import ChangePasswordForm, PasswordResetForm, PasswordResetTokenForm
-from account.forms import SettingsForm
+from account.forms import (
+    ChangePasswordForm,
+    LoginUsernameForm,
+    PasswordResetForm,
+    PasswordResetTokenForm,
+    SettingsForm,
+    SignupForm,
+)
 from account.hooks import hookset
 from account.mixins import LoginRequiredMixin
-from account.models import SignupCode, EmailAddress, EmailConfirmation, Account, AccountDeletion
-from account.utils import default_redirect
+from account.models import (
+    Account,
+    AccountDeletion,
+    EmailAddress,
+    EmailConfirmation,
+    PasswordHistory,
+    SignupCode,
+)
+from account.utils import default_redirect, get_form_data, is_ajax
 
 
-class SignupView(FormView):
+class PasswordMixin(object):
+    """
+    Mixin handling common elements of password change.
+
+    Required attributes in inheriting class:
+
+      form_password_field - example: "password"
+      fallback_url_setting - example: "ACCOUNT_PASSWORD_RESET_REDIRECT_URL"
+
+    Required methods in inheriting class:
+
+      get_user()
+      change_password()
+      after_change_password()
+      get_redirect_field_name()
+
+    """
+
+    redirect_field_name = "next"
+    messages = {
+        "password_changed": {
+            "level": messages.SUCCESS,
+            "text": _("Password successfully changed.")
+        }
+    }
+
+    def get_context_data(self, **kwargs):
+        ctx = super(PasswordMixin, self).get_context_data(**kwargs)
+        redirect_field_name = self.get_redirect_field_name()
+        ctx.update({
+            "redirect_field_name": redirect_field_name,
+            "redirect_field_value": self.request.POST.get(redirect_field_name, self.request.GET.get(redirect_field_name, "")),
+        })
+        return ctx
+
+    def change_password(self, form):
+        user = self.get_user()
+        user.set_password(form.cleaned_data[self.form_password_field])
+        user.save()
+        return user
+
+    def after_change_password(self):
+        user = self.get_user()
+        signals.password_changed.send(sender=self, user=user)
+        if settings.ACCOUNT_NOTIFY_ON_PASSWORD_CHANGE:
+            self.send_password_email(user)
+        if self.messages.get("password_changed"):
+            messages.add_message(
+                self.request,
+                self.messages["password_changed"]["level"],
+                self.messages["password_changed"]["text"]
+            )
+
+    def get_redirect_field_name(self):
+        return self.redirect_field_name
+
+    def get_success_url(self, fallback_url=None, **kwargs):
+        if fallback_url is None:
+            fallback_url = getattr(settings, self.fallback_url_setting, None)
+        kwargs.setdefault("redirect_field_name", self.get_redirect_field_name())
+        return default_redirect(self.request, fallback_url, **kwargs)
+
+    def send_password_email(self, user):
+        protocol = settings.ACCOUNT_DEFAULT_HTTP_PROTOCOL
+        current_site = get_current_site(self.request)
+        ctx = {
+            "user": user,
+            "protocol": protocol,
+            "current_site": current_site,
+        }
+        hookset.send_password_change_email([user.email], ctx)
+
+    def create_password_history(self, form, user):
+        if settings.ACCOUNT_PASSWORD_USE_HISTORY:
+            password = form.cleaned_data[self.form_password_field]
+            PasswordHistory.objects.create(
+                user=user,
+                password=make_password(password)
+            )
+
+
+class SignupView(PasswordMixin, FormView):
 
     template_name = "account/signup.html"
     template_name_ajax = "account/ajax/signup.html"
@@ -34,6 +130,7 @@ class SignupView(FormView):
     template_name_signup_closed_ajax = "account/ajax/signup_closed.html"
     form_class = SignupForm
     form_kwargs = {}
+    form_password_field = "password"
     redirect_field_name = "next"
     identifier_field = "username"
     messages = {
@@ -46,12 +143,16 @@ class SignupView(FormView):
             "text": _("The code {code} is invalid.")
         }
     }
+    fallback_url_setting = "ACCOUNT_SIGNUP_REDIRECT_URL"
 
     def __init__(self, *args, **kwargs):
         self.created_user = None
         kwargs["signup_code"] = None
         super(SignupView, self).__init__(*args, **kwargs)
 
+    @method_decorator(sensitive_post_parameters())
+    @method_decorator(csrf_protect)
+    @method_decorator(never_cache)
     def dispatch(self, request, *args, **kwargs):
         self.request = request
         self.args = args
@@ -72,13 +173,15 @@ class SignupView(FormView):
             self.signup_code_present = False
 
     def get(self, *args, **kwargs):
-        if self.request.user.is_authenticated():
+        if self.request.user.is_authenticated:
             return redirect(default_redirect(self.request, settings.ACCOUNT_LOGIN_REDIRECT_URL))
         if not self.is_open():
             return self.closed()
         return super(SignupView, self).get(*args, **kwargs)
 
     def post(self, *args, **kwargs):
+        if self.request.user.is_authenticated:
+            raise Http404()
         if not self.is_open():
             return self.closed()
         return super(SignupView, self).post(*args, **kwargs)
@@ -92,20 +195,11 @@ class SignupView(FormView):
         return initial
 
     def get_template_names(self):
-        if self.request.is_ajax():
+        if is_ajax(self.request):
             return [self.template_name_ajax]
         else:
             return [self.template_name]
 
-    def get_context_data(self, **kwargs):
-        ctx = kwargs
-        redirect_field_name = self.get_redirect_field_name()
-        ctx.update({
-            "redirect_field_name": redirect_field_name,
-            "redirect_field_value": self.request.REQUEST.get(redirect_field_name, ""),
-        })
-        return ctx
-
     def get_form_kwargs(self):
         kwargs = super(SignupView, self).get_form_kwargs()
         kwargs.update(self.form_kwargs)
@@ -114,8 +208,8 @@ class SignupView(FormView):
     def form_invalid(self, form):
         signals.user_sign_up_attempt.send(
             sender=SignupForm,
-            username=form.data.get("username"),
-            email=form.data.get("email"),
+            username=get_form_data(form, self.identifier_field),
+            email=get_form_data(form, "email"),
             result=form.is_valid()
         )
         return super(SignupView, self).form_invalid(form)
@@ -132,6 +226,7 @@ class SignupView(FormView):
             self.created_user.is_active = False
             self.created_user.save()
         self.create_account(form)
+        self.create_password_history(form, self.created_user)
         self.after_signup(form)
         if settings.ACCOUNT_EMAIL_CONFIRMATION_EMAIL and not email_address.verified:
             self.send_email_confirmation(email_address)
@@ -158,17 +253,11 @@ class SignupView(FormView):
             self.login_user()
         return redirect(self.get_success_url())
 
-    def get_success_url(self, fallback_url=None, **kwargs):
-        if fallback_url is None:
-            fallback_url = settings.ACCOUNT_SIGNUP_REDIRECT_URL
-        kwargs.setdefault("redirect_field_name", self.get_redirect_field_name())
-        return default_redirect(self.request, fallback_url, **kwargs)
-
-    def get_redirect_field_name(self):
-        return self.redirect_field_name
-
-    def create_user(self, form, commit=True, **kwargs):
-        user = get_user_model()(**kwargs)
+    def create_user(self, form, commit=True, model=None, **kwargs):
+        User = model
+        if User is None:
+            User = get_user_model()
+        user = User(**kwargs)
         username = form.cleaned_data.get("username")
         if username is None:
             username = self.generate_username(form)
@@ -196,7 +285,7 @@ class SignupView(FormView):
         kwargs.setdefault("primary", True)
         kwargs.setdefault("verified", False)
         if self.signup_code:
-            kwargs["verified"] = self.signup_code.email and self.created_user.email == self.signup_code.email
+            kwargs["verified"] = self.created_user.email == self.signup_code.email if self.signup_code.email else False
         return EmailAddress.objects.add_email(self.created_user, self.created_user.email, **kwargs)
 
     def use_signup_code(self, user):
@@ -210,16 +299,9 @@ class SignupView(FormView):
         signals.user_signed_up.send(sender=SignupForm, user=self.created_user, form=form)
 
     def login_user(self):
-        user = self.created_user
-        if settings.ACCOUNT_USE_AUTH_AUTHENTICATE:
-            # call auth.authenticate to ensure we set the correct backend for
-            # future look ups using auth.get_user().
-            user = auth.authenticate(**self.user_credentials())
-        else:
-            # set auth backend to ModelBackend, but this may not be used by
-            # everyone. this code path is deprecated and will be removed in
-            # favor of using auth.authenticate above.
-            user.backend = "django.contrib.auth.backends.ModelBackend"
+        user = auth.authenticate(**self.user_credentials())
+        if not user:
+            raise ImproperlyConfigured("Configured auth backends failed to authenticate on signup")
         auth.login(self.request, user)
         self.request.session.set_expiry(0)
 
@@ -227,7 +309,7 @@ class SignupView(FormView):
         return hookset.get_user_credentials(self.form, self.identifier_field)
 
     def get_code(self):
-        return self.request.REQUEST.get("code")
+        return self.request.POST.get("code", self.request.GET.get("code"))
 
     def is_open(self):
         if self.signup_code:
@@ -245,7 +327,7 @@ class SignupView(FormView):
         return settings.ACCOUNT_OPEN_SIGNUP
 
     def email_confirmation_required_response(self):
-        if self.request.is_ajax():
+        if is_ajax(self.request):
             template_name = self.template_name_email_confirmation_sent_ajax
         else:
             template_name = self.template_name_email_confirmation_sent
@@ -260,7 +342,7 @@ class SignupView(FormView):
         return self.response_class(**response_kwargs)
 
     def closed(self):
-        if self.request.is_ajax():
+        if is_ajax(self.request):
             template_name = self.template_name_signup_closed_ajax
         else:
             template_name = self.template_name_signup_closed
@@ -279,23 +361,29 @@ class LoginView(FormView):
     form_kwargs = {}
     redirect_field_name = "next"
 
+    @method_decorator(sensitive_post_parameters())
+    @method_decorator(csrf_protect)
+    @method_decorator(never_cache)
+    def dispatch(self, *args, **kwargs):
+        return super(LoginView, self).dispatch(*args, **kwargs)
+
     def get(self, *args, **kwargs):
-        if self.request.user.is_authenticated():
+        if self.request.user.is_authenticated:
             return redirect(self.get_success_url())
         return super(LoginView, self).get(*args, **kwargs)
 
     def get_template_names(self):
-        if self.request.is_ajax():
+        if is_ajax(self.request):
             return [self.template_name_ajax]
         else:
             return [self.template_name]
 
     def get_context_data(self, **kwargs):
-        ctx = kwargs
+        ctx = super(LoginView, self).get_context_data(**kwargs)
         redirect_field_name = self.get_redirect_field_name()
         ctx.update({
             "redirect_field_name": redirect_field_name,
-            "redirect_field_value": self.request.REQUEST.get(redirect_field_name, ""),
+            "redirect_field_value": self.request.POST.get(redirect_field_name, self.request.GET.get(redirect_field_name, "")),
         })
         return ctx
 
@@ -307,7 +395,7 @@ class LoginView(FormView):
     def form_invalid(self, form):
         signals.user_login_attempt.send(
             sender=LoginView,
-            username=form.data.get(form.identifier_field),
+            username=get_form_data(form, form.identifier_field),
             result=form.is_valid()
         )
         return super(LoginView, self).form_invalid(form)
@@ -340,14 +428,18 @@ class LogoutView(TemplateResponseMixin, View):
     template_name = "account/logout.html"
     redirect_field_name = "next"
 
+    @method_decorator(never_cache)
+    def dispatch(self, *args, **kwargs):
+        return super(LogoutView, self).dispatch(*args, **kwargs)
+
     def get(self, *args, **kwargs):
-        if not self.request.user.is_authenticated():
+        if not self.request.user.is_authenticated:
             return redirect(self.get_redirect_url())
         ctx = self.get_context_data()
         return self.render_to_response(ctx)
 
     def post(self, *args, **kwargs):
-        if self.request.user.is_authenticated():
+        if self.request.user.is_authenticated:
             auth.logout(self.request)
         return redirect(self.get_redirect_url())
 
@@ -356,7 +448,7 @@ class LogoutView(TemplateResponseMixin, View):
         redirect_field_name = self.get_redirect_field_name()
         ctx.update({
             "redirect_field_name": redirect_field_name,
-            "redirect_field_value": self.request.REQUEST.get(redirect_field_name, ""),
+            "redirect_field_value": self.request.POST.get(redirect_field_name, self.request.GET.get(redirect_field_name, "")),
         })
         return ctx
 
@@ -377,6 +469,10 @@ class ConfirmEmailView(TemplateResponseMixin, View):
         "email_confirmed": {
             "level": messages.SUCCESS,
             "text": _("You have confirmed {email}.")
+        },
+        "email_confirmation_expired": {
+            "level": messages.ERROR,
+            "text": _("Email confirmation for {email} has expired.")
         }
     }
 
@@ -393,17 +489,30 @@ class ConfirmEmailView(TemplateResponseMixin, View):
 
     def post(self, *args, **kwargs):
         self.object = confirmation = self.get_object()
-        confirmation.confirm()
-        self.after_confirmation(confirmation)
-        redirect_url = self.get_redirect_url()
-        if not redirect_url:
-            ctx = self.get_context_data()
-            return self.render_to_response(ctx)
-        if self.messages.get("email_confirmed"):
+        self.user = self.request.user
+        confirmed = confirmation.confirm() is not None
+        if confirmed:
+            self.after_confirmation(confirmation)
+            if settings.ACCOUNT_EMAIL_CONFIRMATION_AUTO_LOGIN:
+                self.user = self.login_user(confirmation.email_address.user)
+            redirect_url = self.get_redirect_url()
+            if not redirect_url:
+                ctx = self.get_context_data()
+                return self.render_to_response(ctx)
+            if self.messages.get("email_confirmed"):
+                messages.add_message(
+                    self.request,
+                    self.messages["email_confirmed"]["level"],
+                    self.messages["email_confirmed"]["text"].format(**{
+                        "email": confirmation.email_address.email
+                    })
+                )
+        else:
+            redirect_url = self.get_redirect_url()
             messages.add_message(
                 self.request,
-                self.messages["email_confirmed"]["level"],
-                self.messages["email_confirmed"]["text"].format(**{
+                self.messages["email_confirmation_expired"]["level"],
+                self.messages["email_confirmation_expired"]["text"].format(**{
                     "email": confirmation.email_address.email
                 })
             )
@@ -428,7 +537,7 @@ class ConfirmEmailView(TemplateResponseMixin, View):
         return ctx
 
     def get_redirect_url(self):
-        if self.request.user.is_authenticated():
+        if self.user.is_authenticated:
             if not settings.ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL:
                 return settings.ACCOUNT_LOGIN_REDIRECT_URL
             return settings.ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL
@@ -440,8 +549,13 @@ class ConfirmEmailView(TemplateResponseMixin, View):
         user.is_active = True
         user.save()
 
+    def login_user(self, user):
+        user.backend = "django.contrib.auth.backends.ModelBackend"
+        auth.login(self.request, user)
+        return user
 
-class ChangePasswordView(FormView):
+
+class ChangePasswordView(PasswordMixin, FormView):
 
     template_name = "account/password_change.html"
     form_class = ChangePasswordForm
@@ -452,36 +566,27 @@ class ChangePasswordView(FormView):
             "text": _("Password successfully changed.")
         }
     }
+    form_password_field = "password_new"
+    fallback_url_setting = "ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL"
 
     def get(self, *args, **kwargs):
-        if not self.request.user.is_authenticated():
+        if not self.request.user.is_authenticated:
             return redirect("account_password_reset")
         return super(ChangePasswordView, self).get(*args, **kwargs)
 
     def post(self, *args, **kwargs):
-        if not self.request.user.is_authenticated():
+        if not self.request.user.is_authenticated:
             return HttpResponseForbidden()
         return super(ChangePasswordView, self).post(*args, **kwargs)
 
-    def change_password(self, form):
-        user = self.request.user
-        user.set_password(form.cleaned_data["password_new"])
-        user.save()
-        # required on Django >= 1.7 to keep the user authenticated
-        if hasattr(auth, "update_session_auth_hash"):
-            auth.update_session_auth_hash(self.request, user)
+    def form_valid(self, form):
+        self.change_password(form)
+        self.create_password_history(form, self.request.user)
+        self.after_change_password()
+        return redirect(self.get_success_url())
 
-    def after_change_password(self):
-        user = self.request.user
-        signals.password_changed.send(sender=ChangePasswordView, user=user)
-        if settings.ACCOUNT_NOTIFY_ON_PASSWORD_CHANGE:
-            self.send_email(user)
-        if self.messages.get("password_changed"):
-            messages.add_message(
-                self.request,
-                self.messages["password_changed"]["level"],
-                self.messages["password_changed"]["text"]
-            )
+    def get_user(self):
+        return self.request.user
 
     def get_form_kwargs(self):
         """
@@ -495,38 +600,11 @@ class ChangePasswordView(FormView):
             })
         return kwargs
 
-    def form_valid(self, form):
-        self.change_password(form)
-        self.after_change_password()
-        return redirect(self.get_success_url())
-
-    def get_context_data(self, **kwargs):
-        ctx = kwargs
-        redirect_field_name = self.get_redirect_field_name()
-        ctx.update({
-            "redirect_field_name": redirect_field_name,
-            "redirect_field_value": self.request.REQUEST.get(redirect_field_name, ""),
-        })
-        return ctx
-
-    def get_redirect_field_name(self):
-        return self.redirect_field_name
-
-    def get_success_url(self, fallback_url=None, **kwargs):
-        if fallback_url is None:
-            fallback_url = settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL
-        kwargs.setdefault("redirect_field_name", self.get_redirect_field_name())
-        return default_redirect(self.request, fallback_url, **kwargs)
-
-    def send_email(self, user):
-        protocol = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http")
-        current_site = get_current_site(self.request)
-        ctx = {
-            "user": user,
-            "protocol": protocol,
-            "current_site": current_site,
-        }
-        hookset.send_password_change_email([user.email], ctx)
+    def change_password(self, form):
+        user = super(ChangePasswordView, self).change_password(form)
+        # required on Django >= 1.7 to keep the user authenticated
+        if hasattr(auth, "update_session_auth_hash"):
+            auth.update_session_auth_hash(self.request, user)
 
 
 class PasswordResetView(FormView):
@@ -536,8 +614,12 @@ class PasswordResetView(FormView):
     form_class = PasswordResetForm
     token_generator = default_token_generator
 
+    @method_decorator(csrf_protect)
+    def dispatch(self, *args, **kwargs):
+        return super(PasswordResetView, self).dispatch(*args, **kwargs)
+
     def get_context_data(self, **kwargs):
-        context = kwargs
+        context = super(PasswordResetView, self).get_context_data(**kwargs)
         if self.request.method == "POST" and "resend" in self.request.POST:
             context["resend"] = True
         return context
@@ -553,7 +635,7 @@ class PasswordResetView(FormView):
 
     def send_email(self, email):
         User = get_user_model()
-        protocol = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http")
+        protocol = settings.ACCOUNT_DEFAULT_HTTP_PROTOCOL
         current_site = get_current_site(self.request)
         email_qs = EmailAddress.objects.filter(email__iexact=email)
         for user in User.objects.filter(pk__in=email_qs.values("user")):
@@ -562,81 +644,73 @@ class PasswordResetView(FormView):
             password_reset_url = "{0}://{1}{2}".format(
                 protocol,
                 current_site.domain,
-                reverse("account_password_reset_token", kwargs=dict(uidb36=uid, token=token))
+                reverse(settings.ACCOUNT_PASSWORD_RESET_TOKEN_URL, kwargs=dict(uidb36=uid, token=token))
             )
             ctx = {
                 "user": user,
                 "current_site": current_site,
                 "password_reset_url": password_reset_url,
             }
-            hookset.send_password_reset_email([user.email], ctx)
+            hookset.send_password_reset_email([email], ctx)
 
     def make_token(self, user):
         return self.token_generator.make_token(user)
 
 
-class PasswordResetTokenView(FormView):
+INTERNAL_RESET_URL_TOKEN = "set-password"
+INTERNAL_RESET_SESSION_TOKEN = "_password_reset_token"
+
+
+class PasswordResetTokenView(PasswordMixin, FormView):
 
     template_name = "account/password_reset_token.html"
     template_name_fail = "account/password_reset_token_fail.html"
     form_class = PasswordResetTokenForm
     token_generator = default_token_generator
-    redirect_field_name = "next"
-    messages = {
-        "password_changed": {
-            "level": messages.SUCCESS,
-            "text": _("Password successfully changed.")
-        },
-    }
+    form_password_field = "password"
+    fallback_url_setting = "ACCOUNT_PASSWORD_RESET_REDIRECT_URL"
+
+    @method_decorator(sensitive_post_parameters())
+    @method_decorator(never_cache)
+    def dispatch(self, *args, **kwargs):
+        user = self.get_user()
+        if user is not None:
+            token = kwargs["token"]
+            if token == INTERNAL_RESET_URL_TOKEN:
+                session_token = self.request.session.get(INTERNAL_RESET_SESSION_TOKEN, "")
+                if self.check_token(user, session_token):
+                    return super(PasswordResetTokenView, self).dispatch(*args, **kwargs)
+            else:
+                if self.check_token(user, token):
+                    # Store the token in the session and redirect to the
+                    # password reset form at a URL without the token. That
+                    # avoids the possibility of leaking the token in the
+                    # HTTP Referer header.
+                    self.request.session[INTERNAL_RESET_SESSION_TOKEN] = token
+                    redirect_url = self.request.path.replace(token, INTERNAL_RESET_URL_TOKEN)
+                    return redirect(redirect_url)
+        return self.token_fail()
 
     def get(self, request, **kwargs):
         form_class = self.get_form_class()
         form = self.get_form(form_class)
         ctx = self.get_context_data(form=form)
-        if not self.check_token(self.get_user(), self.kwargs["token"]):
-            return self.token_fail()
         return self.render_to_response(ctx)
 
     def get_context_data(self, **kwargs):
-        ctx = kwargs
-        redirect_field_name = self.get_redirect_field_name()
+        ctx = super(PasswordResetTokenView, self).get_context_data(**kwargs)
         ctx.update({
             "uidb36": self.kwargs["uidb36"],
             "token": self.kwargs["token"],
-            "redirect_field_name": redirect_field_name,
-            "redirect_field_value": self.request.REQUEST.get(redirect_field_name, ""),
         })
         return ctx
 
-    def change_password(self, form):
-        user = self.get_user()
-        user.set_password(form.cleaned_data["password"])
-        user.save()
-
-    def after_change_password(self):
-        user = self.get_user()
-        signals.password_changed.send(sender=PasswordResetTokenView, user=user)
-        if self.messages.get("password_changed"):
-            messages.add_message(
-                self.request,
-                self.messages["password_changed"]["level"],
-                self.messages["password_changed"]["text"]
-            )
-
     def form_valid(self, form):
         self.change_password(form)
+        self.create_password_history(form, self.get_user())
         self.after_change_password()
         return redirect(self.get_success_url())
 
-    def get_redirect_field_name(self):
-        return self.redirect_field_name
-
-    def get_success_url(self, fallback_url=None, **kwargs):
-        if fallback_url is None:
-            fallback_url = settings.ACCOUNT_PASSWORD_RESET_REDIRECT_URL
-        kwargs.setdefault("redirect_field_name", self.get_redirect_field_name())
-        return default_redirect(self.request, fallback_url, **kwargs)
-
     def get_user(self):
         try:
             uid_int = base36_to_int(self.kwargs["uidb36"])
@@ -712,11 +786,11 @@ class SettingsView(LoginRequiredMixin, FormView):
                 self.primary_email_address.change(email, confirm=confirm)
 
     def get_context_data(self, **kwargs):
-        ctx = kwargs
+        ctx = super(SettingsView, self).get_context_data(**kwargs)
         redirect_field_name = self.get_redirect_field_name()
         ctx.update({
             "redirect_field_name": redirect_field_name,
-            "redirect_field_value": self.request.REQUEST.get(redirect_field_name, ""),
+            "redirect_field_value": self.request.POST.get(redirect_field_name, self.request.GET.get(redirect_field_name, "")),
         })
         return ctx
 
@@ -742,7 +816,7 @@ class SettingsView(LoginRequiredMixin, FormView):
         return default_redirect(self.request, fallback_url, **kwargs)
 
 
-class DeleteView(LogoutView):
+class DeleteView(LoginRequiredMixin, LogoutView):
 
     template_name = "account/delete.html"
     messages = {
@@ -765,7 +839,7 @@ class DeleteView(LogoutView):
         return redirect(self.get_redirect_url())
 
     def get_context_data(self, **kwargs):
-        ctx = super(DeleteView, self).get_context_data()
+        ctx = super(DeleteView, self).get_context_data(**kwargs)
         ctx.update(kwargs)
         ctx["ACCOUNT_DELETION_EXPUNGE_HOURS"] = settings.ACCOUNT_DELETION_EXPUNGE_HOURS
         return ctx
